@@ -1,4 +1,5 @@
-/* vim: set sw=4 sts=4: -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
+/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* vim: set sw=4 sts=4 ts=4 expandtab: */
 /* 
    rsvg-css.c: Parse CSS basic data types.
  
@@ -93,8 +94,8 @@ typedef enum _RelativeSize {
 } RelativeSize;
 
 static double
-rsvg_css_parse_raw_length (const char *str, gint * in,
-                           gint * percent, gint * em, gint * ex, RelativeSize * relative_size)
+rsvg_css_parse_raw_length (const char *str, gboolean * in,
+                           gboolean * percent, gboolean * em, gboolean * ex, RelativeSize * relative_size)
 {
     double length = 0.0;
     char *p = NULL;
@@ -177,7 +178,7 @@ RsvgLength
 _rsvg_css_parse_length (const char *str)
 {
     RsvgLength out;
-    gint percent, em, ex, in;
+    gboolean percent, em, ex, in;
     RelativeSize relative_size = RELATIVE_SIZE_NORMAL;
     percent = em = ex = in = FALSE;
 
@@ -252,37 +253,6 @@ _rsvg_css_hand_normalize_length (const RsvgLength * in, gdouble pixels_per_inch,
     return 0;
 }
 
-gboolean
-rsvg_css_param_match (const char *str, const char *param_name)
-{
-    int i;
-
-    for (i = 0; str[i] != '\0' && param_name[i] != '\0'; i++)
-        if (param_name[i] != str[i])
-            return FALSE;
-    return str[i] == ':' && param_name[i] == '\0';
-}
-
-int
-rsvg_css_param_arg_offset (const char *str)
-{
-    int i;
-    int found;
-    found = -1;
-    i = 0;
-    while (str[i] != '\0') {
-        for (; str[i] != '\0' && str[i] != ':'; i++);
-        if (str[i] != '\0')
-            i++;
-        for (; str[i] == ' '; i++);
-        if (str[i] != '\0')
-            found = i;
-    }
-    if (found == -1)
-        found = i;
-    return found;
-}
-
 static gint
 rsvg_css_clip_rgb_percent (gdouble in_percent)
 {
