@@ -1,4 +1,5 @@
-/* vim: set sw=4 sts=4: -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
+/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* vim: set sw=4 sts=4 ts=4 expandtab: */
 /*
    rsvg-gobject.c: GObject support.
 
@@ -53,7 +54,10 @@ instance_init (RsvgHandle * self)
     self->priv->dpi_x = rsvg_internal_dpi_x;
     self->priv->dpi_y = rsvg_internal_dpi_y;
 
-    self->priv->css_props = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
+    self->priv->css_props = g_hash_table_new_full (g_str_hash,
+                                                   g_str_equal,
+                                                   g_free,
+                                                   (GDestroyNotify) g_hash_table_destroy);
 
     self->priv->ctxt = NULL;
     self->priv->currentnode = NULL;
@@ -81,7 +85,7 @@ rsvg_ctx_free_helper (gpointer key, gpointer value, gpointer user_data)
     xmlFree (entval->orig);
     xmlFree (entval);
 #else
-	xmlFreeNode(entval);
+    xmlFreeNode((xmlNode *) entval);
 #endif
 }
 
@@ -94,7 +98,7 @@ instance_dispose (GObject * instance)
 
 #if HAVE_SVGZ
     if (self->priv->is_gzipped)
-        g_object_unref (G_OBJECT (self->priv->gzipped_data));
+        g_object_unref (self->priv->gzipped_data);
 #endif
 
     g_hash_table_foreach (self->priv->entities, rsvg_ctx_free_helper, NULL);
@@ -116,7 +120,7 @@ instance_dispose (GObject * instance)
 
     g_free (self->priv);
 
-    rsvg_parent_class->dispose (G_OBJECT (self));
+    rsvg_parent_class->dispose (instance);
 }
 
 static void
@@ -198,9 +202,9 @@ class_init (RsvgHandleClass * klass)
     gobject_class->set_property = set_property;
     gobject_class->get_property = get_property;
 
-        /**
-	 * dpi-x:
-	 */
+    /**
+     * dpi-x:
+     */
     g_object_class_install_property (gobject_class,
                                      PROP_DPI_X,
                                      g_param_spec_double ("dpi-x", _("Horizontal resolution"),
@@ -313,7 +317,7 @@ rsvg_handle_get_type (void)
 void
 rsvg_handle_free (RsvgHandle * handle)
 {
-    g_object_unref (G_OBJECT (handle));
+    g_object_unref (handle);
 }
 
 /**
