@@ -1,4 +1,5 @@
-/* vim: set sw=4 sts=4: -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
+/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* vim: set sw=4 sts=4 ts=4 expandtab: */
 /*
    rsvg-structure.c: Rsvg's structual elements
 
@@ -47,11 +48,28 @@ rsvg_node_draw (RsvgNode * self, RsvgDrawingCtx * ctx, int dominate)
             return;
         ctx->drawsub_stack = stacksave->next;
     }
-    if (0 /*!state->visible */ )
+    if (!state->visible)
         return;
 
+    if (g_slist_find(ctx->ptrs, self) != NULL)
+    {
+        /*
+         * 5.3.1 of the SVG 1.1 spec (http://www.w3.org/TR/SVG11/struct.html#HeadOverview)
+         * seems to indicate ("URI references that directly or indirectly reference
+         * themselves are treated as invalid circular references") that circular
+         * references are invalid, and so we can drop them to avoid infinite recursion.
+         * 
+         * See also http://bugzilla.gnome.org/show_bug.cgi?id=518640
+         */
+        g_warning("Circular SVG reference noticed, dropping");
+        return;
+    }
+    ctx->ptrs = g_slist_append(ctx->ptrs, self);
+
     self->draw (self, ctx, dominate);
     ctx->drawsub_stack = stacksave;
+
+    ctx->ptrs = g_slist_remove(ctx->ptrs, self);
 }
 
 /* generic function for drawing all of the children of a particular node */
@@ -87,7 +105,7 @@ _rsvg_node_dont_set_atts (RsvgNode * node, RsvgHandle * ctx, RsvgPropertyBag * a
 void
 _rsvg_node_init (RsvgNode * self)
 {
-	self->parent = NULL;
+    self->parent = NULL;
     self->children = g_ptr_array_new ();
     self->state = g_new (RsvgState, 1);
     rsvg_state_init (self->state);
@@ -108,7 +126,6 @@ _rsvg_node_finalize (RsvgNode * self)
         g_ptr_array_free (self->children, TRUE);
     if (self->type != NULL)
         g_string_free (self->type, TRUE);
-
 }
 
 void
@@ -317,7 +334,7 @@ rsvg_node_svg_draw (RsvgNode * self, RsvgDrawingCtx * ctx, int dominate)
 static void
 rsvg_node_svg_set_atts (RsvgNode * self, RsvgHandle * ctx, RsvgPropertyBag * atts)
 {
-    const char *id = NULL, *klazz = NULL, *value;
+    const char *value;
     RsvgNodeSvg *svg = (RsvgNodeSvg *) self;
 
     if (rsvg_property_bag_size (atts)) {
@@ -330,22 +347,35 @@ rsvg_node_svg_set_atts (RsvgNode * self, RsvgHandle * ctx, RsvgPropertyBag * att
             svg->w = _rsvg_css_parse_length (value);
         if ((value = rsvg_property_bag_lookup (atts, "height")))
             svg->h = _rsvg_css_parse_length (value);
-
-		/* 
-		 * x & y attributes have no effect on outermost svg
-		 * http://www.w3.org/TR/SVG/struct.html#SVGElement 
-		 */
+        /* 
+         * x & y attributes have no effect on outermost svg
+         * http://www.w3.org/TR/SVG/struct.html#SVGElement 
+         */
         if (self->parent && (value = rsvg_property_bag_lookup (atts, "x")))
             svg->x = _rsvg_css_parse_length (value);
         if (self->parent && (value = rsvg_property_bag_lookup (atts, "y")))
             svg->y = _rsvg_css_parse_length (value);
-        if ((value = rsvg_property_bag_lookup (atts, "class")))
-            klazz = value;
         if ((value = rsvg_property_bag_lookup (atts, "id"))) {
-            id = value;
             rsvg_defs_register_name (ctx->priv->defs, value, &svg->super);
         }
-        rsvg_parse_style_attrs (ctx, self->state, "svg", klazz, id, atts);
+        /*
+         * style element is not loaded yet here, so we need to store those attribues
+         * to be applied later.
+         */
+        svg->atts = rsvg_property_bag_ref(atts);
+    }
+}
+
+void
+_rsvg_node_svg_apply_atts (RsvgNodeSvg * self, RsvgHandle * ctx)
+{
+    const char *id = NULL, *klazz = NULL, *value;
+    if (rsvg_property_bag_size (self->atts)) {
+        if ((value = rsvg_property_bag_lookup (self->atts, "class")))
+            klazz = value;
+        if ((value = rsvg_property_bag_lookup (self->atts, "id")))
+            id = value;
+        rsvg_parse_style_attrs (ctx, ((RsvgNode *)self)->state, "svg", klazz, id, self->atts);
     }
 }
 
@@ -396,7 +426,7 @@ rsvg_node_use_set_atts (RsvgNode * self, RsvgHandle * ctx, RsvgPropertyBag * att
 }
 
 RsvgNode *
-rsvg_new_use ()
+rsvg_new_use (void)
 {
     RsvgNodeUse *use;
     use = g_new (RsvgNodeUse, 1);
@@ -450,7 +480,7 @@ rsvg_new_symbol (void)
 }
 
 RsvgNode *
-rsvg_new_defs ()
+rsvg_new_defs (void)
 {
     RsvgNodeGroup *group;
     group = g_new (RsvgNodeGroup, 1);
