@@ -1,4 +1,5 @@
-/* vim: set sw=4 sts=4: -*- Mode: C; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
+/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* vim: set sw=4 sts=4 ts=4 expandtab: */
 /*
    rsvg-text.c: Text handling routines for RSVG
 
@@ -104,30 +105,30 @@ _rsvg_text_chomp (RsvgState *state, GString * in, gboolean * lastwasspace)
     guint i;
     out = g_string_new (in->str);
 
-	if (!state->space_preserve) {
-		for (i = 0; i < out->len;) {
-			if (out->str[i] == '\n')
-				g_string_erase (out, i, 1);
-			else
-				i++;
-		}
-		
-		for (i = 0; i < out->len; i++)
-			if (out->str[i] == '\t')
-				out->str[i] = ' ';
-		
-		for (i = 0; i < out->len;) {
-			if (out->str[i] == ' ' && *lastwasspace)
-				g_string_erase (out, i, 1);
-			else {
-				if (out->str[i] == ' ')
-					*lastwasspace = TRUE;
-				else
-					*lastwasspace = FALSE;
-				i++;
-			}
-		}
-	}
+    if (!state->space_preserve) {
+        for (i = 0; i < out->len;) {
+            if (out->str[i] == '\n')
+                g_string_erase (out, i, 1);
+            else
+                i++;
+        }
+
+        for (i = 0; i < out->len; i++)
+            if (out->str[i] == '\t')
+                out->str[i] = ' ';
+
+        for (i = 0; i < out->len;) {
+            if (out->str[i] == ' ' && *lastwasspace)
+                g_string_erase (out, i, 1);
+            else {
+                if (out->str[i] == ' ')
+                    *lastwasspace = TRUE;
+                else
+                    *lastwasspace = FALSE;
+                i++;
+            }
+        }
+    }
 
     return out;
 }
@@ -216,6 +217,8 @@ _rsvg_node_text_length_children (RsvgNode * self, RsvgDrawingCtx * ctx,
     int out = FALSE;
     for (i = 0; i < self->children->len; i++) {
         RsvgNode *node = g_ptr_array_index (self->children, i);
+        rsvg_state_push (ctx);
+        rsvg_state_reinherit_top (ctx, node->state, 0);
         if (!strcmp (node->type->str, "RSVG_NODE_CHARS")) {
             RsvgNodeChars *chars = (RsvgNodeChars *) node;
             GString *str = _rsvg_text_chomp (rsvg_state_current (ctx), chars->contents, lastwasspace);
@@ -228,6 +231,7 @@ _rsvg_node_text_length_children (RsvgNode * self, RsvgDrawingCtx * ctx,
             RsvgNodeTref *tref = (RsvgNodeTref *) node;
             out = _rsvg_node_text_length_tref (tref, ctx, x, lastwasspace);
         }
+        rsvg_state_pop (ctx);
         if (out)
             break;
     }
@@ -508,7 +512,7 @@ rsvg_text_get_pango_context (RsvgDrawingCtx * ctx)
 static void
 rsvg_text_layout_free (RsvgTextLayout * layout)
 {
-    g_object_unref (G_OBJECT (layout->layout));
+    g_object_unref (layout->layout);
     g_free (layout);
 }
 
@@ -588,8 +592,12 @@ rsvg_text_layout_new (RsvgDrawingCtx * ctx, RsvgState * state, const char *text)
 {
     RsvgTextLayout *layout;
 
-    if (ctx->pango_context == NULL)
-        ctx->pango_context = rsvg_text_get_pango_context (ctx);
+    if (ctx->pango_context == NULL) {
+        if (ctx->render->create_pango_context)
+            ctx->pango_context = ctx->render->create_pango_context (ctx);
+        else
+            ctx->pango_context = rsvg_text_get_pango_context (ctx);
+    }
 
     layout = g_new0 (RsvgTextLayout, 1);
 
@@ -783,7 +791,7 @@ rsvg_text_render_vectors (PangoFont * font,
     face = pango_ft2_font_get_face (font);
 
     if (0 != FT_Load_Glyph (face, (FT_UInt) pango_glyph, flags))
-		return;
+        return;
 
     if (0 != FT_Get_Glyph (face->glyph, &glyph))
 		return;
@@ -929,6 +937,11 @@ rsvg_text_render_text (RsvgDrawingCtx * ctx, const char *text, gdouble * x, gdou
         gint w, h, baseline;
 
         state = rsvg_state_current (ctx);
+
+        /* Do not render the text if the font size is zero. See bug #581491. */
+        if (state->font_size.length == 0)
+            return;
+
         context = ctx->render->create_pango_context (ctx);
         layout = rsvg_text_create_layout (ctx, state, text, context);
         pango_layout_get_size (layout, &w, &h);
